﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Xml;
using Win8App.ContactService;
using Windows.Foundation;
using Windows.UI.Popups;

namespace Win8App
{
    public class GroupedContact : ObservableCollection<Contact>
    {
        public GroupedContact(IEnumerable<Contact> data)
            :base(data)
        {
            
        }
        public string HeaderInfo { get; set; }
    }

    public class ViewModel : INotifyPropertyChanged
    {
        private static string Url = "http://localhost:4472/ContactService.svc";
        private ContactServiceClient _client;

        public ViewModel()
        {
            SayHiCommand = new Command<object>(OnSayHigh);
            PopupMenuCommand = new UICommand(OnPopupMenuExecuted, "Test", "testcommand");
            ShowMenuCommand = new Command<object>(OnShowMenu);
        }

        private void OnPopupMenuExecuted(IUICommand command)
        {
            Debug.WriteLine("You clicked on UI command with id of " + command.Id, "Win 8");
        }

        public async Task Initialize()
        {
            IsBusy = true;
            ConfigureCLient();
            Contacts = await _client.GetContactsAsync();
            var query = from c in Contacts
                        group c by c.LastName.Substring(0, 1)
                        into lgroup
                        select new
                            {
                                HeaderInfo = lgroup.Key,
                                Contacts = lgroup
                            };
            GroupedContacts = new ObservableCollection<GroupedContact>();
            foreach (var VARIABLE in query)
            {
                GroupedContacts.Add(new GroupedContact(VARIABLE.Contacts)
                    {
                        HeaderInfo = VARIABLE.HeaderInfo
                    });
            }
            IsBusy = false;
        }


        private ObservableCollection<Contact> _contacts;
        public ObservableCollection<Contact> Contacts
        {
            get { return _contacts; }
            set { _contacts = value; OnPropertyChanged("Contacts"); }
        }

        private ObservableCollection<GroupedContact> _grouped;
        public ObservableCollection<GroupedContact> GroupedContacts
        {
            get { return _grouped; }
            set { _grouped = value; OnPropertyChanged("GroupedContacts"); }
        }

        private bool _isBusy;

        public bool IsBusy
        {
            get { return _isBusy; }
            set { _isBusy = value; OnPropertyChanged("IsBusy"); }
        }
        

        private void ConfigureCLient()
        {
            var binding = new BasicHttpBinding();
            binding.ReaderQuotas = XmlDictionaryReaderQuotas.Max;
            _client = new ContactServiceClient(binding, new EndpointAddress(Url));
        }

        public Command<object> SayHiCommand { get; set; }

        public Command<object> ShowMenuCommand { get; set; }

        public async void OnShowMenu(object parameter)
        {
           PopupMenu m = new PopupMenu();
           m.Commands.Add(PopupMenuCommand);
            m.ShowAsync(new Point(200, 200));
        }


        public async void OnSayHigh(object parameter)
        {
            var dialog = new MessageDialog("Hi");
            dialog.Commands.Add(PopupMenuCommand);
            await dialog.ShowAsync();

        }

        public UICommand PopupMenuCommand { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }


    }
}
